@@ -1,5 +1,3 @@
-(* $Id$ *)
-
 (* included: type.ml *)
 
 open Bi_outbuf
@@ -126,7 +124,7 @@ let float_needs_period s =
 
 (*
   Both write_float_fast and write_float guarantee
-  that a sufficient number of digits are printed in order to 
+  that a sufficient number of digits are printed in order to
   allow reversibility.
 
   The _fast version is faster but often produces unnecessarily long numbers.
@@ -142,7 +140,7 @@ let write_float_fast ob x =
       Bi_outbuf.add_string ob s;
       if float_needs_period s then
 	Bi_outbuf.add_string ob ".0"
-	
+
 let write_float ob x =
   match classify_float x with
     FP_nan ->
@@ -159,6 +157,40 @@ let write_float ob x =
       if float_needs_period s then
 	Bi_outbuf.add_string ob ".0"
 
+let write_normal_float_prec significant_figures ob x =
+  let open Printf in
+  let s =
+    match significant_figures with
+        1 -> sprintf "%.1g" x
+      | 2 -> sprintf "%.2g" x
+      | 3 -> sprintf "%.3g" x
+      | 4 -> sprintf "%.4g" x
+      | 5 -> sprintf "%.5g" x
+      | 6 -> sprintf "%.6g" x
+      | 7 -> sprintf "%.7g" x
+      | 8 -> sprintf "%.8g" x
+      | 9 -> sprintf "%.9g" x
+      | 10 -> sprintf "%.10g" x
+      | 11 -> sprintf "%.11g" x
+      | 12 -> sprintf "%.12g" x
+      | 13 -> sprintf "%.13g" x
+      | 14 -> sprintf "%.14g" x
+      | 15 -> sprintf "%.15g" x
+      | 16 -> sprintf "%.16g" x
+      | _ -> sprintf "%.17g" x
+  in
+  Bi_outbuf.add_string ob s;
+  if float_needs_period s then
+    Bi_outbuf.add_string ob ".0"
+
+let write_float_prec significant_figures ob x =
+  match classify_float x with
+    FP_nan ->
+      Bi_outbuf.add_string ob "NaN"
+  | FP_infinite ->
+      Bi_outbuf.add_string ob (if x > 0. then "Infinity" else "-Infinity")
+  | _ ->
+      write_normal_float_prec significant_figures ob x
 
 let json_string_of_float x =
   let ob = Bi_outbuf.create 20 in
@@ -166,13 +198,12 @@ let json_string_of_float x =
   Bi_outbuf.contents ob
 
 
-
 let write_std_float_fast ob x =
   match classify_float x with
     FP_nan ->
       json_error "NaN value not allowed in standard JSON"
   | FP_infinite ->
-      json_error 
+      json_error
 	(if x > 0. then
 	   "Infinity value not allowed in standard JSON"
 	 else
@@ -182,13 +213,13 @@ let write_std_float_fast ob x =
       Bi_outbuf.add_string ob s;
       if float_needs_period s then
 	Bi_outbuf.add_string ob ".0"
-	
+
 let write_std_float ob x =
   match classify_float x with
     FP_nan ->
       json_error "NaN value not allowed in standard JSON"
   | FP_infinite ->
-      json_error 
+      json_error
 	(if x > 0. then
 	   "Infinity value not allowed in standard JSON"
 	 else
@@ -202,7 +233,20 @@ let write_std_float ob x =
       Bi_outbuf.add_string ob s;
       if float_needs_period s then
 	Bi_outbuf.add_string ob ".0"
-	
+
+let write_std_float_prec significant_figures ob x =
+  match classify_float x with
+    FP_nan ->
+      json_error "NaN value not allowed in standard JSON"
+  | FP_infinite ->
+      json_error
+	(if x > 0. then
+	   "Infinity value not allowed in standard JSON"
+	 else
+	   "-Infinity value not allowed in standard JSON")
+  | _ ->
+      write_normal_float_prec significant_figures ob x
+
 let std_json_string_of_float x =
   let ob = Bi_outbuf.create 20 in
   write_std_float ob x;
@@ -412,7 +456,16 @@ let to_channel ?buf ?len ?std oc x =
   in
   to_outbuf ?std ob x;
   Bi_outbuf.flush_channel_writer ob
-  
+
+let to_output ?buf ?len ?std out x =
+  let ob =
+    match buf with
+	None -> Bi_outbuf.create_output_writer ?len out
+      | Some ob -> ob
+  in
+  to_outbuf ?std ob x;
+  Bi_outbuf.flush_output_writer ob
+
 let to_file ?len ?std file x =
   let oc = open_out file in
   try
@@ -446,7 +499,7 @@ let stream_to_channel ?buf ?len ?std oc st =
   in
   stream_to_outbuf ?std ob st;
   Bi_outbuf.flush_channel_writer ob
-  
+
 let stream_to_file ?len ?std file st =
   let oc = open_out file in
   try
@@ -455,3 +508,23 @@ let stream_to_file ?len ?std file st =
   with e ->
     close_out_noerr oc;
     raise e
+
+
+let rec sort = function
+  | `Assoc l ->
+      let l = List.rev (List.rev_map (fun (k, v) -> (k, sort v)) l) in
+      `Assoc (List.stable_sort (fun (a, _) (b, _) -> String.compare a b) l)
+  | `List l ->
+      `List (List.rev (List.rev_map sort l))
+#ifdef TUPLE
+  | `Tuple l ->
+      `Tuple (List.rev (List.rev_map sort l))
+#endif
+#ifdef VARIANT
+  | `Variant (k, Some v) as x ->
+      let v' = sort v in
+      if v == v' then x
+      else
+        `Variant (k, Some v')
+#endif
+  | x -> x
