@@ -1,5 +1,3 @@
-(* $Id$ *)
-
 (** {2 JSON writers} *)
 
 val to_string :
@@ -33,6 +31,20 @@ val to_channel :
 
       See [to_string] for the role of the other optional arguments. *)
 
+val to_output :
+  ?buf:Bi_outbuf.t ->
+  ?len:int ->
+  ?std:bool ->
+  < output : string -> int -> int -> int; .. > -> json -> unit
+  (** Write a compact JSON value to an OO channel.
+      @param buf allows to reuse an existing buffer created with 
+      [Bi_outbuf.create_output_writer] on the same channel.
+      [buf] is flushed right
+      before [to_output] returns but the channel itself is
+      not flushed automatically.
+
+      See [to_string] for the role of the other optional arguments. *)
+
 val to_file :
   ?len:int ->
   ?std:bool ->
@@ -80,6 +92,12 @@ val stream_to_outbuf :
       an existing buffer.
       See [to_string] for the role of the optional arguments. *)
 
+(** {2 Miscellaneous} *)
+
+val sort : json -> json
+  (** Sort object fields (stable sort, comparing field names
+      and treating them as byte sequences) *)
+
 
 
 (**/**)
@@ -95,6 +113,8 @@ val write_float : Bi_outbuf.t -> float -> unit
 val write_std_float : Bi_outbuf.t -> float -> unit
 val write_float_fast : Bi_outbuf.t -> float -> unit
 val write_std_float_fast : Bi_outbuf.t -> float -> unit
+val write_float_prec : int -> Bi_outbuf.t -> float -> unit
+val write_std_float_prec : int -> Bi_outbuf.t -> float -> unit
 #endif
 #ifdef STRING
 val write_string : Bi_outbuf.t -> string -> unit
